`timescale 100ps/100ps
module HAMSI_TOP(
	clk, rst_n,                    /* clock and reset */
	init, EOM, load, fetch, idata, /* input from Control FPGA */
	ack, odata,                    /* output for Control FPGA */
	led                            /* LED for debug */
);   

/* Input/Output */
input         clk, rst_n;
input         init, EOM, load, fetch;
input  [15:0] idata;
output        ack;
output [15:0] odata;
output  [7:0] led;

/* register for debug */
reg debug0, debug1, debug2, debug3, debug4, debug5, debug6, debug7;

/* wire */
/* HAMSI_INTERFACE -> HAMSI_CORE */
wire        init_r, EOM_r, EN;
wire [31:0] idata_r;

/* HAMSI_CORE -> HAMSI_INTERFACE */
wire        busy;
wire [31:0] hash0;
wire [31:0] hash1;
wire [31:0] hash2;
wire [31:0] hash3;
wire [31:0] hash4;
wire [31:0] hash5;
wire [31:0] hash6;
wire [31:0] hash7;

/* LED */
wire [7:0] led;

/* led for debug (when msg = 6162 6380 0000 0000 0000 0018)*/
always @(posedge clk or negedge rst_n) begin
   if(~rst_n) debug0 <= 0;
   else if (hash0 == 32'h6b017b90) debug0 <= 1;
   else debug0 <= debug0;
end
assign led[0] = debug0;

always @(posedge clk or negedge rst_n) begin
   if(~rst_n) debug1 <= 0;
   else if (hash1 == 32'h971fdb64) debug1 <= 1;
   else debug1 <= debug1;
end
assign led[1] = debug1;

always @(posedge clk or negedge rst_n) begin
   if(~rst_n) debug2 <= 0;
   else if (hash2 == 32'h6700dea0) debug2 <= 1;
   else debug2 <= debug2;
end
assign led[2] = debug2;

always @(posedge clk or negedge rst_n) begin
   if(~rst_n) debug3 <= 0;
   else if (hash3 == 32'he50e7ac1) debug3 <= 1;
   else debug3 <= debug3;
end
assign led[3] = debug3;


always @(posedge clk or negedge rst_n) begin
   if(~rst_n) debug4 <= 0;
   else if (hash4 == 32'hf6a75a84) debug4 <= 1;
   else debug4 <= debug4;
end
assign led[4] = debug4;


always @(posedge clk or negedge rst_n) begin
   if(~rst_n) debug5 <= 0;
   else if (hash5 == 32'h9b2809a5) debug5 <= 1;
   else debug5 <= debug5;
end
assign led[5] = debug5;


always @(posedge clk or negedge rst_n) begin
   if(~rst_n) debug6 <= 0;
   else if (hash6 == 32'h5eedcde4) debug6 <= 1;
   else debug6 <= debug6;
end
assign led[6] = debug6;


always @(posedge clk or negedge rst_n) begin
   if(~rst_n) debug7 <= 0;
   else if (hash7 == 32'hc65daf1f) debug7 <= 1;
   else debug7 <= debug7;
end
assign led[7] = debug7;

/* instance */
HAMSI_INTERFACE hamsi_interface(
   .clk(clk), .rst_n(rst_n),
   .init(init), .EOM(EOM), .load(load), .fetch(fetch), .idata(idata),
   .ack(ack), .odata(odata),
   .busy(busy), .hash0(hash0), .hash1(hash1), .hash2(hash2), .hash3(hash3), .hash4(hash4), .hash5(hash5), .hash6(hash6), .hash7(hash7),
   .init_r(init_r), .EOM_r(EOM_r), .EN(EN), .idata_r(idata_r)
);

HAMSI_CORE hamsi_core(
   .clk(clk), .rst_n(rst_n),
   .init_r(init_r), .EOM_r(EOM_r), .EN(EN), .idata_r(idata_r),
   .busy(busy), .hash0(hash0), .hash1(hash1), .hash2(hash2), .hash3(hash3), .hash4(hash4), .hash5(hash5), .hash6(hash6), .hash7(hash7)
);

endmodule
